import PropTypes from "prop-types";
import React from "react";
import { View, StyleSheet, Text } from "react-native";
import { diffChars, diffWords, diffSentences } from "diff";

const fnMap = {
  chars: diffChars,
  words: diffWords,
  sentences: diffSentences,
};

export default function Diff({
  inputA = "",
  inputB = "",
  type = "chars",
  textStyle = {},
  viewStyle = {},
  addedText = {},
  removedText = {},
  unchangedText = {},
  containerStyle = {},
  addedTextView = {},
  removedTextView = {},
}) {
  var diff = fnMap[type](inputA, inputB);
  var result = diff.map(function (part, index) {
    var spanStyle = part.added
      ? { ...styles.addedText, ...addedText }
      : part.removed
      ? { ...styles.removedText, ...removedText }
      : { ...styles.defaultText, ...unchangedText };

    var computedViewStyle = part.added
      ? { ...styles.addedTextView, ...addedTextView }
      : part.removed
      ? { ...styles.removedTextView, ...removedTextView }
      : { ...unchangedTextView };

    return (
      <View key={index} style={[computedViewStyle, viewStyle]}>
        <Text style={[styles.defaultText, spanStyle, textStyle]}>
          {part.value}
        </Text>
      </View>
    );
  });
  return (
    <View style={[styles.defaultContainerStyle, containerStyle]}>{result}</View>
  );
}

Diff.propTypes = {
  inputA: PropTypes.oneOfType([PropTypes.string]),
  inputB: PropTypes.oneOfType([PropTypes.string]),
  type: PropTypes.oneOf(["chars", "words", "sentences"]),
  textStyle: PropTypes.object,
  containerStyle: PropTypes.object,
  addedText: PropTypes.object,
  removedText: PropTypes.object,
  unchangedText: PropTypes.object,
  addedTextView: PropTypes.object,
  removedTextView: PropTypes.object,
  unchangedTextView: PropTypes.object,
};

var styles = StyleSheet.create({
  defaultText: {},
  addedTextView: {
    backgroundColor: "lightgreen",
  },
  addedText: {
    color: "#000000",
  },
  removedTextView: {
    backgroundColor: "salmon",
  },
  removedText: {
    textDecorationLine: "line-through",
    textDecorationStyle: "solid",
    color: "#000000",
  },
  defaultContainerStyle: {
    flexDirection: "row",
    flexWrap: "wrap",
    alignItems: "flex-end",
  },
});
